with Ada.Text_IO;

with Museum;

package body Tourists is

   --  we need a modular type, bounded to Limit
   --+ for task instances to keep count of their
   --+ subsequent visits to the Museum
   Limit : constant Positive := 10;
   type Counter_T is mod Limit;

   --+---------------------------
   task body Visitor_T is
      use Ada.Text_IO;
      use Museum;
      --  each process instance will keep a running
      --+ counter of the number of visits paid to
      --+ the very beautiful museum guarded by
      --+ Museum.Guardian
      --+
      --+ to avoid bothering about possible overflow
      --+ of counter increments, we'd rather use a
      --+ variable of modular type (Counter_T)
      Visit : Counter_T := 0;
   begin
      loop
         Visit := Visit + 1;
         Put_Line ("Visitor #" & Positive'Image (Id)
                   & " queuing for its visit nr. "
                   & Counter_T'Image (Visit)
                   & " to the Museum.");
         --  the process applies for entrance
         Guardian.Entrance;
         --  permission is granted
         Put_Line ("Visitor #" & Positive'Image (Id)
                   & " was granted permission to enter.");
         --  at this point, the process visits, sleeps,
         --+ dines with the dining philosophers and then
         --+ repeats the whole cycle
         delay Duration (Interval);
      end loop;
   end Visitor_T;

end Tourists;
